import fire

from .main import run, activate, deactivate, settings
from .logger import stream
from .util import enable_w1


fire.Fire({
    'run': run,
    'log': run,
    'settings': settings,
    'activate': activate,
    'start': activate,
    'deactivate': deactivate,
    'stop': deactivate,
    'stream': stream,
    'enable_w1': enable_w1
})



import os
import re
import glob
from datetime import datetime as dt

from .util import get_serial_number
from raspi_logger import keywords


def _get_sensors(path):
    return glob.glob(path + '28-*')


def _get_temperature(sensor_path):
    with open(sensor_path + '/w1_slave', 'r') as f: 
        c = f.read().split('\n')
    
    m = re.match(r"([0-9a-f]{2} ){9}t=([+-]?[0-9]+)", c[1])
    if m:
        value = float(m.group(2)) / 1000.
    else:
        value = 'NaN'

    return value, '\n'.join(c)


def read_sensor(path='/sys/bus/w1/devices/', omit_sensor=False, omit_keyword=False, conf={}):
    data = []

    # get the Raspi serial number
    versions = get_serial_number()

    # get sensor config
    sensors = conf.get('sensors', {})

    for p in _get_sensors(path):
        temperature, hextemp = _get_temperature(p)

        d = dict(
            value=temperature,
            tstamp=dt.now().isoformat(),
            identifier=os.path.basename(p),
            rawData=hextemp,
            sensorName='DS18B20'
        )

        # extend
        if not omit_keyword:
            # get the sensor config
            if p in sensors:
                extra = sensors[p]
            elif '_all_' in sensors:
                extra = sensors['_all_']
            else:
                extra = {}
            in_soil = extra.get("in_soil", False)
        
            # add the correct variable and sensor information
            _uuid = keywords.SOIL_TEMPERATURE if in_soil else keywords.AIR_TEMPERATURE
            variable = dict(
            variableName='SOIL TEMPERATURE' if in_soil else 'AIR TEMPERATURE',
                gcmdURL=keywords.CONCEPT_URL.format(uuid=_uuid, fmt='xml'),
                gcmdUUID=_uuid
            )

            # update
            d.update(variable)

        if not omit_sensor:
            d.update(versions)            

        data.append(d)
    
    # return
    return data


if __name__ == '__main__':
    import fire
    fire.Fire(read_sensor)



from setuptools import setup, find_packages


def version():
    with open('VERSION', 'r') as f:
        return f.read().strip()


def readme():
    with open('README.md', 'r') as f:
        return f.read()


def requirements():
    with open('requirements.txt', 'r') as f:
        return f.read().split('\n')



setup(
    name='raspi_logger',
    version=version(),
    author='Mirko Mälicke',
    author_email='mirko@hydrocode.de',
    description='Raspberry Pi data logging software toolkit',
    long_description=readme(),
    long_description_content_type='text/markdown',
    install_requires=requirements(),
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'raspi_logger = raspi_logger.__main__' 
        ]
    },
    zip_safe=False
)


import os
from os.path import join as pjoin
import json
from datetime import datetime as dt
from time import time, sleep
from crontab import CronTab

from .ds18b20 import read_sensor
from .util import parse_interval_to_seconds, config
from .sqlite_backend import append_data


def _save_json_backend(path, new_data, conf):
    # create a new file per day
    if path is None:
        date = dt.now().date()
        fname = '%d_%d_%d_raw_log.json' % (date.year, date.month, date.day)

        # get path
        path = pjoin(conf.get('loggerPath', pjoin(os.path.expanduser('~'), 'logger')), fname)

    if not os.path.exists(os.path.dirname(path)):
        os.makedirs(os.path.dirname(path))

    # save data
    # get existing data
    try:
        with open(path, 'r') as f:
            old_data = json.load(f)
    except:
        old_data = []
    
    # append and save
    with open(path, 'w') as js:
        old_data.extend(new_data)
        json.dump(old_data, js, indent=4)


def save_data(path=None, dry=False, **kwargs):
    # get the config
    conf = config()

    # read the sensor data
    # TODO, here the protocols could be loaded from config.
    data = read_sensor(conf=conf, **kwargs)
    
    # in dry runs, only return the data
    if dry:
        return data

    if len(data) == 0:
        print('[ERROR]: recieved no data. Need better logging. Sorry.')
        return []
    
    # check the registered backends
    for name, c in conf.get('loggerBackends', {}).items():
        # check if the backend is currently enabled
        if c.get('enabled', True):
            if name == 'json':
                _save_json_backend(path, data, conf)
            elif name == 'sqlite':
                append_data(data, conf, path)

    # return the data for reuse
    return data


def stream(interval=None, dry=False, **kwargs):
    # get the start time
    t1 = time()
    
    if interval is None:
        interval = config().get('loggerInterval', '1min')
    else:
        config(loggerInterval=interval)
    
    if isinstance(interval, str):
        interval = parse_interval_to_seconds(interval)
    
    data = save_data(dry=dry, **kwargs)

    # stringify
    outstr = json.dumps(data, indent=4)

    # print
    print(outstr)

    # sleep for the remaining time
    remain = interval - (time() - t1)
    if remain < 0:
        remain = 0
    sleep(remain)

    # call again
    stream(dry=dry, **kwargs)


if __name__=='__main__':
    import fire
    fire.Fire({
        'save': save_data,
        'stream': stream,
    })
